const jwtSecret   = 'SniPPetValeraZhgi'

const express           = require('express');
const express_graphql   = require('express-graphql');

const { buildSchema, printSchema } = require('graphql');
const expand = require('mm-graphql/expand')
const fs     = require('fs')
const uploadPath = `${__dirname}/public/images/`;
const upload  = require('multer')({ dest: uploadPath })


;(async () => {

    const {Savable, slice, getModels} = await require('./models.js')()
    const { jwtGQL, jwtCheck } = require('mm-graphql/jwt')

    const {anonSchema, anonResolvers} = require('./anon')({Savable, secret: jwtSecret})

    let schema = buildSchema(`
        type User {
             _id: String
             createdAt: String
             login: String
             nick : String
             avatar: Image
             theme: String
             fontSize: Int
        }

        input UserInput {
             _id: String
             login: String
             nick : String
             avatar: ImageInput
             theme: String
             fontSize: Int
        }

        type Image {
            _id: ID,
            text: String,
            createdAt: String
            url: String,
            originalFileName: String,
            userAvatar: User,
            owner: User
        }

        input ImageInput {
            _id: ID,
            createdAt: String
            text: String,
            userAvatar: UserInput
        }

        type Snippet {
            _id: ID,
            owner: User
            createdAt: String

            title: String
            description: String,

            comments: [Comment]
            files: [File]
            tags: [String]
        }

        input SnippetInput {
            _id: ID,

            title: String
            description: String,

            comments: [CommentInput]
            files: [FileInput]
            tags: [String]
        }


        type Comment {
            _id: ID,
            owner: User
            createdAt: String

            text: String
            snippet: Snippet
            answers: [Comment]
            answerTo: Comment
        }

        input CommentInput {
            _id: ID,
            text: String

            snippet: SnippetInput
            answerTo: CommentInput
        }

        type File {
            _id: ID,
            owner: User
            createdAt: String

            name: String,
            text: String
            type: String

            snippet: Snippet
        }

        input FileInput {
            _id: ID,

            name: String,
            text: String
            type: String

            snippet: SnippetInput
        }
    `);

    schema = expand(schema)
    console.log(printSchema(schema))

    const app = express();
    app.use(express.static('public'));
    app.use('/graphql', express_graphql(jwtGQL({anonSchema, anonResolvers, schema, createContext: getModels, graphiql: true, secret: jwtSecret})))


    app.post('/upload', upload.single('photo'), async (req, res, next) => {
        let decoded;
        console.log('wtf')
        if (decoded = jwtCheck(req, jwtSecret)){
            console.log('SOME UPLOAD', decoded, req.file)

            let {models: {Image }} = await getModels(decoded.sub)
            let image = await Image.fromFileData(req.file)
		if (image){
			res.end(JSON.stringify({_id: image._id, url: image.url}))
		}
		else {
			res.end('YAKAS CHUHNYA')
		}
        }
        else {
            res.status(503).send('permission denied')
        }
    })


    app.use(express.static('public'));


    let socketPath = "/home/asmer/node_hosts/snippet"
    app.listen(socketPath, () => {
        console.log('Express GraphQL Server Now Running On localhost:4000/graphql');
        fs.chmodSync(socketPath, '777');
    });
})()

